function [Gradient,Hessian] = Gradient_Hessian(Rho,fC,Meas,psiHat)
% Computes the gradient and Hessian of F at point Rho
%
% Copyright 2015, Armin Eftekhari and Michael Wakin
%
% This file is part of the Greedy Super-Resolution Toolbox version 1.0.
%
%    The Greedy Super-Resolution Toolbox is free software: you can 
%    redistribute it and/or modify it under the terms of the GNU General 
%    Public License as published by the Free Software Foundation, either 
%    version 3 of the License, or (at your option) any later version.
%
%    The Greedy Super-Resolution Toolbox is distributed in the hope that it 
%    will be useful, but WITHOUT ANY WARRANTY; without even the implied 
%    warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  
%    See the GNU General Public License for more details.
%
%    You should have received a copy of the GNU General Public License
%    along with the Greedy Super-Resolution Toolbox.  If not, 
%    see <http://www.gnu.org/licenses/>.
%

N = 2*fC+1;
e = @(rho,f) exp(-1i*2*pi*[-f:f]'*rho');
G_rho = diag(psiHat)*e(Rho,fC);
L = diag(1i*2*pi*[-fC:fC]);
P_rho = G_rho*pinv(G_rho);
beta_rho = pinv(G_rho)*Meas;


Gradient = -2*diag(beta_rho)*G_rho'*L*(eye(N)-P_rho)*Meas;





% Drho0 = diag(beta_rho);
% term1 = real(-2*(Drho0')*(G_rho')*(L.^2)*G_rho*Drho0 - 2*(Drho0')*diag((G_rho')*(L.^2)*(eye(N)-P_rho)*Meas));
% 
% 
% Hessian = term1  ...
%     -2*real( (Drho0*(G_rho'*L*G_rho) - diag((G_rho')*L*(eye(N) - P_rho)*Meas)))...
%     * real((inv((G_rho')*G_rho))) * real((Drho0*(G_rho'*L*G_rho) - diag((G_rho')*L*(eye(N) - P_rho)*Meas)))';

Drho0 = diag(beta_rho);


Hessian = real(-2*Drho0*(G_rho')*(L.^2)*G_rho*Drho0 ...
    -2*Drho0*diag((G_rho')*(L.^2)*(eye(N)-P_rho)*Meas))  ...
    -2*real( (Drho0*(G_rho'*L*G_rho) - diag((G_rho')*L*(eye(N) - P_rho)*Meas)))...
    * real((inv((G_rho')*G_rho))) * real((Drho0*(G_rho'*L*G_rho) - diag((G_rho')*L*(eye(N) - P_rho)*Meas)))';

